package cd.casic.module.machine.service;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

public interface MachineInfoService {
    /**
     * 新增机器
     * @return 新增机器的id
     */
    Long createMachine(MachineInfoVO MachineInfoVO);

    /**
     * 查看机器列表
     * @return 机器列表
     */
    PageResult<MachineInfoDO> listMachineInfo(@Valid MachineInfoVO MachineInfoVO);

    /**
     * 更新机器信息
     */
    void updateMachineInfo(@Valid MachineInfoVO machineInfoVO);

    /**
     * 变更机器状态
     */
    Integer updateStatus(@Valid MachineInfoVO machineInfoVO);

    /**
     * 绑定密钥
     */
    void bindingSecretKey(SecretKeyVO secretKeyVO);

    /**
     * 批量删除机器
     * @param machineInfoIds 机器id列表
     */
    void deleteMachineInfoList(String machineInfoIds);

    /**
     * 删除机器信息
     * @param machineInfoId 机器id
     */
    void deleteMachineInfo(Long machineInfoId);

    /**
     * 获取绑定的密钥
     * @param secretKeyId 密钥id
     * @return 绑定的机器列表
     */
    List<MachineInfoDO>selectBindMachineBySecretKey(Long secretKeyId);


    /**
     * 测试机器连接
     * @param id 机器id
     * @return 连接是否成功
     */
    boolean testConnection(Long id);

    /**
     * 连接远程机器
     *
     * @param id 机器id
     * @return  连接后端文件树
     */
    Map<String, Object> connect(Long id);

    /**
     * 获取机器连接状态
     *
     * @return 连接状态
     */
    ConnectionStatus getConnectionStatus(Long id);

    /**
     * 获取所有连接状态
     *
     * @return 机器名称到连接状态的映射
     */
    Map<Long, ConnectionStatus> getAllConnectionStatus();

    /**
     * 上传文件到远程机器
     *
     * @param file  上传文件
     * @param remoteFilePath 远程文件路径
     * @return 操作结果
     */
    boolean uploadFile(MultipartFile file, String remoteFilePath);

    /**
     * 从远程机器下载文件
     *
     * @param remoteFilePath 远程文件路径
     * @return 操作结果
     */
    boolean downloadFile(String remoteFilePath, HttpServletResponse httpServletResponse);

    /**
     * 校验机器是否存在
     *
     * @param id 机器id
     */
    MachineInfoDO validateMachineInfoExists(Long id);

    /**
     * 根据路径获得远程文件树
     *
     * @param machineId 机器id
     * @param path 文件夹路径
     * @return 远程文件树
     */
    Map<String, Object> fileTreeNode(Long machineId, String path);

    /**
     * 删除远程文件路径
     * @param remoteFilePath 远程文件路径
     */
    boolean deleteRemoteFile(String remoteFilePath);
}
