package cd.casic.module.machine.service.impl;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.component.FileTreeComponent;
import cd.casic.module.machine.component.WebSocketConnection;
import cd.casic.module.machine.component.WebSocketSessionManager;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.enums.AuthenticationType;
import cd.casic.module.machine.enums.MachineInfoType;
import cd.casic.module.machine.dal.mysql.MachineInfoMapper;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.enums.MachineInfoStatus;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import com.jcraft.jsch.Session;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import cd.casic.framework.commons.util.object.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 机器信息服务实现类
 */
@Slf4j
@Service("machineInfoService")
public class MachineInfoServiceImpl implements MachineInfoService {

    @Resource
    private SecretKeyService secretKeyService;

    @Resource
    private MachineInfoMapper machineInfoMapper;

    @Resource
    private FileTreeComponent fileTreeComponent;

    //todo，部署后更改
    private static final Path TEMP_DIR = Paths.get("D:\\桌面\\work\\ops-pro111\\temp");

    @Override
    public Long createMachine(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        validateMachineTagUnique(machineInfoVO.getTag());
        MachineInfoDO machineInfoDO = BeanUtils.toBean(machineInfoVO, MachineInfoDO.class);
        if (machineInfoVO.getAuthenticationType() == 2) {
            Long secretKeyId = machineInfoDO.getSecretKeyId();
            SecretKeyVO secretKey = secretKeyService.getSecretKey(secretKeyId);
            if (secretKey == null) {
                throw exception(SECRET_KEY_NOT_EXISTS);
            }
        }
        machineInfoMapper.insert(machineInfoDO);
        return machineInfoDO.getId();
    }

    @Override
    public void updateMachineInfo(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        String newTag = machineInfoVO.getTag();
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoVO.getId());
        String oldTag = machineInfoDO.getTag();
        if (!newTag.equals(oldTag)) {
            validateMachineTagUnique(newTag);
        }
        BeanUtils.copyProperties(machineInfoVO, machineInfoDO);
        machineInfoMapper.updateById(machineInfoDO);
    }

    @Override
    public Integer updateStatus(MachineInfoVO machineInfoVO) {
        machineInfoMapper.updateStatus(machineInfoVO.getId(), machineInfoVO.getStatus());
        return machineInfoVO.getStatus();
    }

    @Override
    public PageResult<MachineInfoDO> listMachineInfo(MachineInfoVO machineInfoVO) {
        return machineInfoMapper.selectPage(machineInfoVO);
    }

    @Override
    public void bindingSecretKey(SecretKeyVO secretKeyVO) {
        machineInfoMapper.bindingSecretKey(secretKeyVO.getMachineInfoIds(), secretKeyVO.getId(),secretKeyVO.getEnableBind());
    }

    @Override
    @Transactional//其中一个在线，那么就回滚
    public void deleteMachineInfoList(String machineInfoIds) {
        List<Long> machineInfoIdList = Arrays.stream(machineInfoIds.split(","))
            .map(String::trim)
            .filter(s -> !s.isEmpty())
            .map(Long::parseLong)
            .toList();
        machineInfoIdList.forEach(this::deleteMachineInfo);
    }

    @Override
    public void deleteMachineInfo(Long machineInfoId) {
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoId);
        validateMachineEnable(machineInfoDO);
        machineInfoMapper.deleteById(machineInfoId);
    }

    @Override
    public List<MachineInfoDO> selectBindMachineBySecretKey(Long secretKeyId) {
        return machineInfoMapper.selectBindMachineBySecretKey(secretKeyId);
    }


    @Override
    public boolean testConnection(Long id) {
        //先查询机器是否存在，在判断机器可用性
        MachineInfoDO machineInfoDO = validateMachineInfoExists(id);
        validateMachineUnEnable(machineInfoDO);
        log.info("测试机器连接: {}", machineInfoDO.getHostIp());
        WebSocketConnection webSocketConnection = createWebSocketConnection(machineInfoDO);
        webSocketConnection.initConnection(machineInfoDO);
        return true;
    }

    @Override
    public Map<String, Object> connect(Long id) {
        //todo使用代理机器的情况还未完成
        WebSocketConnection webSocketConnection = new WebSocketConnection(this.secretKeyService);
        MachineInfoDO machineInfoDO = validateMachineInfoExists(id);
        //初始化连接
        webSocketConnection.initConnection(machineInfoDO);
        WebSocketSessionManager.addWebSocketConnectionByMachineId(id, webSocketConnection);
        return fileTreeComponent.getRemoteFileTree(webSocketConnection.getSshSession(), "/");
    }

    @Override
    public ConnectionStatus getConnectionStatus(Long id) {
        validateMachineInfoExists(id);
        WebSocketConnection webSocketConnection = WebSocketSessionManager.getWebSocketConnectionByMachineId(id);
        return webSocketConnection == null ? ConnectionStatus.DISCONNECTED : webSocketConnection.getConnectionStatus();
    }

    @Override
    public Map<Long, ConnectionStatus> getAllConnectionStatus() {
        return WebSocketSessionManager.getAllWebSocketConnections().entrySet().stream()
            .collect(Collectors.toMap(
                Map.Entry::getKey,
                entry -> entry.getValue().getConnectionStatus()
            ));
    }

    @Override
    public boolean uploadFile(MultipartFile file, String remoteFilePath) {
        Path tempFilePath;
        // 保存上传的文件到临时位置
        String originalFilename = file.getOriginalFilename();
        if (originalFilename == null || originalFilename.isEmpty()) {
            throw exception(INVALID_FILE_NAME);
        }
        // 安全过滤文件名，防止路径遍历攻击
        String safeFilename = sanitizeFilename(originalFilename);
        try {
            tempFilePath = Files.createTempFile(TEMP_DIR, "upload-", safeFilename);
            file.transferTo(tempFilePath.toFile());
        } catch (IOException e) {
            throw exception(CREATE_TEMP_FILE_ERROR);
        }
        fileTreeComponent.uploadToRemoteServer(tempFilePath, safeFilename, remoteFilePath);
        return true;
    }

    @Override
    public boolean downloadFile(String remoteFilePath, HttpServletResponse httpServletResponse) {
        fileTreeComponent.downloadFile(remoteFilePath, httpServletResponse);
        return true;
    }

    @VisibleForTesting
    void validateMachineEnvAdd(MachineInfoVO machineInfoVO) {
        if (machineInfoVO.getHostIp().isEmpty()) {
            throw exception(MACHINE_INFO_HOST_IP_NULL);
        }
        if (machineInfoVO.getUsername().isEmpty()) {
            throw exception(MACHINE_INFO_USER_NAME_NULL);
        }
        if (machineInfoVO.getTag().isEmpty()) {
            throw exception(MACHINE_INFO_TAG_NULL);
        }

        if (machineInfoVO.getAuthenticationType() != null) {
            boolean flag = true;
            for (int type : AuthenticationType.ARRAYS) {
                if (type == machineInfoVO.getAuthenticationType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NULL);
        }


        if (machineInfoVO.getMachineInfoType() != null) {
            boolean flag = true;
            for (int type : MachineInfoType.ARRAYS) {
                if (type == machineInfoVO.getMachineInfoType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_TYPE_NULL);
        }

    }

    @VisibleForTesting
    void validateMachineTagUnique(String tag) {
        if (machineInfoMapper.existsByTag(tag)) {
            throw exception(MACHINE_INFO_TAG_EXISTS);
        }
    }

    @Override
    public MachineInfoDO validateMachineInfoExists(Long id) {
        if (id == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        MachineInfoDO machineInfoDO = machineInfoMapper.selectById(id);
        if (machineInfoDO == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        return machineInfoDO;
    }

    @Override
    public Map<String, Object> fileTreeNode(Long machineId, String path) {
        validateMachineInfoExists(machineId);
        Session sshSession = WebSocketSessionManager.getWebSocketConnectionByMachineId(machineId).getSshSession();
        return fileTreeComponent.getRemoteFileTree(sshSession, path);
    }

    @Override
    public boolean deleteRemoteFile(String remoteFilePath) {
        fileTreeComponent.deleteRemoteFile(remoteFilePath);
        return true;
    }

    @VisibleForTesting
    void validateMachineEnable(MachineInfoDO machineInfoDO) {
        if (machineInfoDO.getStatus() == MachineInfoStatus.ENABLE.getCode()) {
            throw exception(MACHINE_ENABLE);
        }
    }

    @VisibleForTesting
    void validateMachineUnEnable(MachineInfoDO machineInfoDO) {

        if (machineInfoDO.getStatus() == MachineInfoStatus.UN_ENABLE.getCode()) {
            throw exception(MACHINE_UN_ENABLE);
        }
    }

    @VisibleForTesting
    WebSocketConnection createWebSocketConnection(MachineInfoDO machineInfoDO) {
        if (WebSocketSessionManager.containsMachineId(machineInfoDO.getId())) {
            return WebSocketSessionManager.getWebSocketConnectionByMachineId((machineInfoDO.getId()));
        } else {
            return new WebSocketConnection(this.secretKeyService);
        }
    }

    @VisibleForTesting
    // 安全过滤文件名，防止路径遍历攻击
    private String sanitizeFilename(String filename) {
        // 移除路径相关字符，只保留文件名和扩展名
        filename = filename.replaceAll("[\\\\/:*?\"<>|]", "_");
        // 限制最大长度
        if (filename.length() > 255) {
            filename = filename.substring(0, 255);
        }

        return filename;
    }
}
